'use strict';

const errors = require('@feathersjs/errors');
const commons = require('@feathersjs/commons');
const makeDebug = require('debug');

function _interopDefaultCompat (e) { return e && typeof e === 'object' && 'default' in e ? e.default : e; }

const makeDebug__default = /*#__PURE__*/_interopDefaultCompat(makeDebug);

var __defProp$h = Object.defineProperty;
var __defNormalProp$h = (obj, key, value) => key in obj ? __defProp$h(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$h = (obj, key, value) => {
  __defNormalProp$h(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class BaseService {
  constructor(options) {
    __publicField$h(this, "stripe");
    __publicField$h(this, "options");
    const opts = {
      // Stripe enforces 100 max and 10 default
      paginate: {
        default: 10,
        max: 100
      },
      ...options
    };
    if (!opts.stripe) {
      throw new Error("Stripe service option `stripe` needs to be provided");
    }
    this.stripe = opts.stripe;
  }
  find(params) {
    if (!this._find) {
      throw new errors.NotImplemented("Find method not implemented");
    }
    return this._find(params).catch(this.handleError);
  }
  get(id, params) {
    if (!this._get) {
      throw new errors.NotImplemented("Get method not implemented");
    }
    return this._get(id, params).catch(this.handleError);
  }
  create(data, params) {
    if (!this._create) {
      throw new errors.NotImplemented("Create method not implemented");
    }
    return this._create(data, params).catch(this.handleError);
  }
  update(id, data, params) {
    if (!this._update) {
      throw new errors.NotImplemented("Update method not implemented");
    }
    return this._update(id, data, params).catch(this.handleError);
  }
  patch(id, data, params) {
    if (!this._patch) {
      throw new errors.NotImplemented("Patch method not implemented");
    }
    return this._patch(id, data, params).catch(this.handleError);
  }
  remove(id, params) {
    if (!this._remove) {
      throw new errors.NotImplemented("Remove method not implemented");
    }
    return this._remove(id, params).catch(this.handleError);
  }
  getLimit(limit, paramsPaginate) {
    if (paramsPaginate === false) {
      return limit;
    }
    const paginate = this.options?.paginate;
    if (paginate && (paginate.default || paginate.max)) {
      const base = paginate.default || 0;
      const lower = typeof limit === "number" && !isNaN(limit) ? limit : base;
      const upper = typeof paginate.max === "number" ? paginate.max : Number.MAX_VALUE;
      return Math.min(lower, upper);
    }
    return limit;
  }
  cleanQuery(query) {
    if (Array.isArray(query)) {
      return query.map((item) => this.cleanQuery(item));
    }
    if (commons._.isObject(query)) {
      const result = Object.assign({}, query);
      Object.entries(result).forEach(([key, value]) => {
        let cleanKey = key;
        if (key.startsWith("$")) {
          delete result[key];
          cleanKey = key.replace("$", "");
        }
        result[cleanKey] = this.cleanQuery(value);
      });
      return result;
    }
    return query;
  }
  filterQuery(params) {
    const query = Object.assign({}, params.query);
    const limit = query.$limit ?? query.limit;
    if (limit) {
      query.limit = this.getLimit(limit, params.paginate);
      delete query.$limit;
    }
    return this.cleanQuery(query);
  }
  filterParams(params) {
    return {
      query: this.filterQuery(params),
      stripe: params.stripe,
      paginate: params.paginate !== false
    };
  }
  async handlePaginate({ paginate }, stripeMethod) {
    if (paginate) {
      return await stripeMethod;
    }
    if (stripeMethod.autoPagingEach) {
      const results = [];
      await stripeMethod.autoPagingEach((result) => {
        results.push(result);
      });
      return results;
    }
    throw new errors.MethodNotAllowed("Cannot use paginate: false on this method");
  }
  handleError(error) {
    if (!error.type) {
      throw new errors.GeneralError("Unknown Payment Gateway Error", error);
    }
    switch (error.type) {
      case "StripeCardError":
        throw new errors.PaymentError(error, error);
      case "StripeInvalidRequestError":
        throw new errors.BadRequest(error, error);
      case "StripeAPIError":
        throw new errors.Unavailable(error, error);
      case "StripeConnectionError":
        throw new errors.Unavailable(error, error);
      case "StripeAuthenticationError":
        throw new errors.NotAuthenticated(error, error);
      case "StripeRateLimitError":
        throw new errors.TooManyRequests(error, error);
      default:
        throw new errors.GeneralError("Unknown Payment Gateway Error", error);
    }
  }
}

var __defProp$g = Object.defineProperty;
var __defNormalProp$g = (obj, key, value) => key in obj ? __defProp$g(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$g = (obj, key, value) => {
  __defNormalProp$g(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class AccountService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$g(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.accounts.list(filtered.query, filtered.stripe)
    );
  }
  async _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.accounts.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.accounts.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.accounts.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp$f = Object.defineProperty;
var __defNormalProp$f = (obj, key, value) => key in obj ? __defProp$f(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$f = (obj, key, value) => {
  __defNormalProp$f(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class AccountLinkService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$f(this, "_find");
    __publicField$f(this, "_get");
    __publicField$f(this, "_update");
    __publicField$f(this, "_patch");
    __publicField$f(this, "_remove");
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.accountLinks.create(data, stripe);
  }
}

var __defProp$e = Object.defineProperty;
var __defNormalProp$e = (obj, key, value) => key in obj ? __defProp$e(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$e = (obj, key, value) => {
  __defNormalProp$e(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug$5 = makeDebug__default("feathers-stripe:application-fee-refund");
class ApplicationFeeRefundService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$e(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    if (!filtered.query.fee) {
      debug$5("Missing Stripe fee id");
    }
    const { fee, ...query } = filtered.query;
    return this.handlePaginate(
      filtered,
      this.stripe.applicationFees.listRefunds(fee, query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    if (!stripe.fee) {
      debug$5("Missing Stripe fee id");
    }
    const { fee, ...rest } = stripe;
    return this.stripe.applicationFees.retrieveRefund(fee, id, rest);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    if (!stripe.fee) {
      debug$5("Missing Stripe fee id");
    }
    const { fee, ...rest } = stripe;
    return this.stripe.applicationFees.createRefund(fee, data, rest);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    if (!stripe.fee) {
      debug$5("Missing Stripe fee id");
    }
    const { fee, ...rest } = stripe;
    return this.stripe.applicationFees.updateRefund(fee, id, data, rest);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp$d = Object.defineProperty;
var __defNormalProp$d = (obj, key, value) => key in obj ? __defProp$d(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$d = (obj, key, value) => {
  __defNormalProp$d(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class BalanceService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$d(this, "_find");
    __publicField$d(this, "_create");
    __publicField$d(this, "_update");
    __publicField$d(this, "_patch");
    __publicField$d(this, "_remove");
  }
  _get(id, params) {
    let { stripe } = this.filterParams(params);
    stripe = Object.assign({}, stripe);
    if (id) {
      stripe.stripeAccount = id;
    }
    return this.stripe.balance.retrieve(void 0, stripe);
  }
}

const debug$4 = makeDebug__default("feathers-stripe:card");
class BankAccountService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    const { customer, ...query } = filtered.query;
    if (!customer) {
      debug$4("Missing Stripe customer id");
    }
    return this.handlePaginate(
      filtered,
      this.stripe.customers.listSources(customer, query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$4("Missing Stripe customer id");
    }
    return this.stripe.customers.retrieveSource(query.customer, id, stripe);
  }
  _create(data, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$4("Missing Stripe customer id");
    }
    return this.stripe.customers.createSource(query.customer, data, stripe);
  }
  _update(id, data, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$4("Missing Stripe customer id");
    }
    return this.stripe.customers.updateSource(query.customer, id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$4("Missing Stripe customer id");
    }
    return this.stripe.customers.deleteSource(query.customer, id, stripe);
  }
}

const debug$3 = makeDebug__default("feathers-stripe:external-account");
class ExternalAccountService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    const { account, ...query } = filtered.query;
    if (!account) {
      debug$3("Missing Stripe account id");
    }
    return this.handlePaginate(
      filtered,
      this.stripe.accounts.listExternalAccounts(account, query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.account) {
      debug$3("Missing Stripe account id");
    }
    return this.stripe.accounts.retrieveExternalAccount(
      query.account,
      id,
      stripe
    );
  }
  // TODO: move 'account' to 'query'?
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    const { account, ...rest } = data;
    if (!account) {
      debug$3("Missing Stripe account id");
    }
    return this.stripe.accounts.createExternalAccount(account, rest, stripe);
  }
  _update(id, data, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.account) {
      debug$3("Missing Stripe account id");
    }
    return this.stripe.accounts.updateExternalAccount(
      query.account,
      id,
      data,
      stripe
    );
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.account) {
      debug$3("Missing Stripe account id");
    }
    return this.stripe.accounts.deleteExternalAccount(
      query.account,
      id,
      stripe
    );
  }
}

const debug$2 = makeDebug__default("feathers-stripe:card");
class CardService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    const { customer, ...query } = filtered.query;
    if (!customer) {
      debug$2("Missing Stripe customer id");
    }
    return this.handlePaginate(
      filtered,
      this.stripe.customers.listSources(customer, query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$2("Missing Stripe customer id");
    }
    return this.stripe.customers.retrieveSource(query.customer, id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    const { customer, ...rest } = data;
    if (!customer) {
      debug$2("Missing Stripe customer id");
    }
    return this.stripe.customers.createSource(customer, rest, stripe);
  }
  _update(id, data, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$2("Missing Stripe customer id");
    }
    return this.stripe.customers.updateSource(query.customer, id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$2("Missing Stripe customer id");
    }
    return this.stripe.customers.deleteSource(query.customer, id, stripe);
  }
}

class ChargeService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.charges.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.charges.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.charges.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.charges.update(id, data, stripe);
  }
  _patch(id, data, params) {
    const { stripe } = this.filterParams(params);
    if ("capture" in data) {
      const { capture, ...rest } = data;
      if (capture) {
        return this.stripe.charges.capture(id, rest, stripe);
      }
    }
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.refunds.create({ charge: id }, stripe);
  }
}

var __defProp$c = Object.defineProperty;
var __defNormalProp$c = (obj, key, value) => key in obj ? __defProp$c(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$c = (obj, key, value) => {
  __defNormalProp$c(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class CouponService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$c(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.coupons.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.coupons.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.coupons.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.coupons.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

class CustomerService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.customers.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.customers.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.customers.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.customers.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.customers.del(id, stripe);
  }
}

var __defProp$b = Object.defineProperty;
var __defNormalProp$b = (obj, key, value) => key in obj ? __defProp$b(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$b = (obj, key, value) => {
  __defNormalProp$b(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class DisputeService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$b(this, "_create");
    __publicField$b(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.disputes.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.disputes.retrieve(id, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.disputes.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp$a = Object.defineProperty;
var __defNormalProp$a = (obj, key, value) => key in obj ? __defProp$a(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$a = (obj, key, value) => {
  __defNormalProp$a(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class EventService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$a(this, "_create");
    __publicField$a(this, "_update");
    __publicField$a(this, "_patch");
    __publicField$a(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.events.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.events.retrieve(id, stripe);
  }
}

var __defProp$9 = Object.defineProperty;
var __defNormalProp$9 = (obj, key, value) => key in obj ? __defProp$9(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$9 = (obj, key, value) => {
  __defNormalProp$9(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class InvoiceService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$9(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.invoices.list(filtered.query, filtered.stripe)
    );
  }
  // TODO (EK): Support upcoming invoices
  // TODO (EK): Support invoice line items
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoices.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoices.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoices.update(id, data, stripe);
  }
  _patch(id, data, params) {
    const { stripe } = this.filterParams(params);
    if ("pay" in data) {
      const { pay } = data;
      if (pay) {
        return this.stripe.invoices.pay(id, stripe);
      }
    }
    return this._update(id, data, params);
  }
}

class InvoiceItemService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.invoiceItems.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoiceItems.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoiceItems.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoiceItems.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.invoiceItems.del(id, stripe);
  }
}

class PayoutService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.payouts.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.payouts.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.payouts.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.payouts.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.payouts.cancel(id, stripe);
  }
}

var __defProp$8 = Object.defineProperty;
var __defNormalProp$8 = (obj, key, value) => key in obj ? __defProp$8(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$8 = (obj, key, value) => {
  __defNormalProp$8(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class PaymentIntentService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$8(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.paymentIntents.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentIntents.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentIntents.create(data, stripe);
  }
  _patch(id, data, params) {
    const { stripe } = this.filterParams(params);
    if ("capture" in data) {
      return this.stripe.paymentIntents.capture(id, stripe);
    }
    return this._update(id, data, params);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentIntents.update(id, data, stripe);
  }
}

class PaymentMethodService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.paymentMethods.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentMethods.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentMethods.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentMethods.update(id, data, stripe);
  }
  _patch(id, data, params) {
    const { stripe } = this.filterParams(params);
    if ("attach" in data) {
      const { attach, ...rest } = data;
      if (attach) {
        return this.stripe.paymentMethods.attach(id, rest, stripe);
      }
    }
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.paymentMethods.detach(id, stripe);
  }
}

class PlanService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.plans.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.plans.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.plans.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.plans.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.plans.del(id, stripe);
  }
}

var __defProp$7 = Object.defineProperty;
var __defNormalProp$7 = (obj, key, value) => key in obj ? __defProp$7(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$7 = (obj, key, value) => {
  __defNormalProp$7(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class PriceService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$7(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.prices.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.prices.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.prices.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.prices.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

class ProductService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.products.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.products.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.products.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.products.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.products.del(id, stripe);
  }
}

var __defProp$6 = Object.defineProperty;
var __defNormalProp$6 = (obj, key, value) => key in obj ? __defProp$6(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$6 = (obj, key, value) => {
  __defNormalProp$6(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class RefundService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$6(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.refunds.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.refunds.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.refunds.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.refunds.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp$5 = Object.defineProperty;
var __defNormalProp$5 = (obj, key, value) => key in obj ? __defProp$5(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$5 = (obj, key, value) => {
  __defNormalProp$5(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class SetupIntentService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$5(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.setupIntents.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.setupIntents.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.setupIntents.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.setupIntents.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp$4 = Object.defineProperty;
var __defNormalProp$4 = (obj, key, value) => key in obj ? __defProp$4(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$4 = (obj, key, value) => {
  __defNormalProp$4(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug$1 = makeDebug__default("feathers-stripe:source");
class SourceService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$4(this, "_find");
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.sources.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    if ("customer" in data) {
      const { customer, ...rest } = data;
      if (customer) {
        return this.stripe.customers.createSource(customer, rest, stripe);
      }
    }
    return this.stripe.sources.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.sources.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.customer) {
      debug$1("Missing Stripe customer id");
    }
    return this.stripe.customers.deleteSource(query.customer, id, stripe);
  }
}

class SubscriptionService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.subscriptions.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptions.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptions.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptions.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptions.cancel(id, stripe);
  }
}

class SubscriptionItemService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.subscriptionItems.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptionItems.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptionItems.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptionItems.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.subscriptionItems.del(id, stripe);
  }
}

var __defProp$3 = Object.defineProperty;
var __defNormalProp$3 = (obj, key, value) => key in obj ? __defProp$3(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$3 = (obj, key, value) => {
  __defNormalProp$3(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class TokenService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$3(this, "_find");
    __publicField$3(this, "_update");
    __publicField$3(this, "_patch");
    __publicField$3(this, "_remove");
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.tokens.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.tokens.create(data, stripe);
  }
}

var __defProp$2 = Object.defineProperty;
var __defNormalProp$2 = (obj, key, value) => key in obj ? __defProp$2(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$2 = (obj, key, value) => {
  __defNormalProp$2(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class TransactionService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$2(this, "_create");
    __publicField$2(this, "_update");
    __publicField$2(this, "_patch");
    __publicField$2(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.balanceTransactions.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.balanceTransactions.retrieve(id, stripe);
  }
}

class TransferService extends BaseService {
  _find(params) {
    const filtered = this.filterParams(params);
    return this.handlePaginate(
      filtered,
      this.stripe.transfers.list(filtered.query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.transfers.retrieve(id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.transfers.create(data, stripe);
  }
  _update(id, data, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.transfers.update(id, data, stripe);
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
  _remove(id, params) {
    const { stripe } = this.filterParams(params);
    return this.stripe.transfers.createReversal(id, stripe);
  }
}

var __defProp$1 = Object.defineProperty;
var __defNormalProp$1 = (obj, key, value) => key in obj ? __defProp$1(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField$1 = (obj, key, value) => {
  __defNormalProp$1(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const debug = makeDebug__default("feathers-stripe:transferReversal");
class TransferReversalService extends BaseService {
  constructor() {
    super(...arguments);
    __publicField$1(this, "_remove");
  }
  _find(params) {
    const filtered = this.filterParams(params);
    const { transfer, ...query } = filtered.query;
    if (!transfer) {
      debug("Missing Stripe transfer id");
    }
    return this.handlePaginate(
      filtered,
      this.stripe.transfers.listReversals(transfer, query, filtered.stripe)
    );
  }
  _get(id, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.transfer) {
      debug("Missing Stripe transfer id");
    }
    return this.stripe.transfers.retrieveReversal(query.transfer, id, stripe);
  }
  _create(data, params) {
    const { stripe } = this.filterParams(params);
    const { transfer, ...rest } = data;
    if (!transfer) {
      debug("Missing Stripe transfer id");
    }
    return this.stripe.transfers.createReversal(transfer, rest, stripe);
  }
  _update(id, data, params) {
    const { query, stripe } = this.filterParams(params);
    if (!query.transfer) {
      debug("Missing Stripe transfer id");
    }
    return this.stripe.transfers.updateReversal(
      query.transfer,
      id,
      data,
      stripe
    );
  }
  _patch(id, data, params) {
    return this._update(id, data, params);
  }
}

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
class WebhookService extends BaseService {
  constructor(options) {
    super(options);
    __publicField(this, "app");
    __publicField(this, "handlers");
    __publicField(this, "isExpressSetup", false);
    __publicField(this, "_find");
    __publicField(this, "_get");
    __publicField(this, "_update");
    __publicField(this, "_patch");
    __publicField(this, "_remove");
    if (!options.app) {
      throw new Error("options.app is required");
    }
    this.app = options.app;
    this.handlers = options.handlers || {};
    this.options = options;
  }
  /**
   * registers an express route
   * @param route
   * @param endpointSecret
   * @returns
   */
  setupExpress(route, endpointSecret) {
    if (this.isExpressSetup) {
      return;
    }
    this.app.post(
      route,
      (req, res, next) => {
        const signature = req.headers["stripe-signature"];
        if (!signature) {
          res.status(400).end("Bad signature");
          return;
        }
        try {
          this.stripe.webhooks.constructEvent(
            // @ts-expect-error - rawBody is not defined in express
            req.rawBody,
            signature,
            endpointSecret
          );
        } catch (err) {
          res.status(400).end(err);
          return;
        }
        next();
      }
    );
    this.isExpressSetup = true;
  }
  getHandler(event) {
    const parts = event.type.split(".");
    let node = this.handlers;
    for (const p of parts) {
      node = node[parts[p]];
      if (!node) {
        return null;
      }
    }
    return node;
  }
  _create(event, params) {
    const handler = this.getHandler(event);
    if (!handler) {
      return Promise.resolve({});
    }
    return handler({ object: event.data.object, event, params, app: this.app });
  }
}
function setupWebhook(app, route, { endpointSecret, stripe, handlers }) {
  const service = new WebhookService({
    handlers,
    stripe,
    app
  });
  service.setupExpress(route, endpointSecret);
  return service;
}

exports.AccountLinkService = AccountLinkService;
exports.AccountService = AccountService;
exports.ApplicationFeeRefundService = ApplicationFeeRefundService;
exports.BalanceService = BalanceService;
exports.BankAccountService = BankAccountService;
exports.CardService = CardService;
exports.ChargeService = ChargeService;
exports.CouponService = CouponService;
exports.CustomerService = CustomerService;
exports.DisputeService = DisputeService;
exports.EventService = EventService;
exports.ExternalAccountService = ExternalAccountService;
exports.InvoiceItemService = InvoiceItemService;
exports.InvoiceService = InvoiceService;
exports.PaymentIntentService = PaymentIntentService;
exports.PaymentMethodService = PaymentMethodService;
exports.PayoutService = PayoutService;
exports.PlanService = PlanService;
exports.PriceService = PriceService;
exports.ProductService = ProductService;
exports.RefundService = RefundService;
exports.SetupIntentService = SetupIntentService;
exports.SourceService = SourceService;
exports.SubscriptionItemService = SubscriptionItemService;
exports.SubscriptionService = SubscriptionService;
exports.TokenService = TokenService;
exports.TransactionService = TransactionService;
exports.TransferReversalService = TransferReversalService;
exports.TransferService = TransferService;
exports.WebhookService = WebhookService;
exports.setupWebhook = setupWebhook;
