import Stripe from 'stripe';
import { PaginationOptions, Params, Query, Application } from '@feathersjs/feathers';

type StripeServiceOptions = {
    paginate?: PaginationOptions;
    stripe: Stripe;
};
type ParamsWithStripe = {
    stripe?: Stripe.RequestOptions;
    [key: string]: any;
};
type ParamsWithStripeFee = {
    stripe: Partial<Stripe.RequestOptions> & {
        fee: string;
    };
    [key: string]: any;
};
type ParamsWithStripeQuery<Q = any> = Params<Q> & {
    stripe?: Stripe.RequestOptions;
    [key: string]: any;
};
interface IUnderScoreFunctions {
    _find: never | ((...args: any) => any);
    _get: never | ((...args: any) => any);
    _create: never | ((data: any, params: any) => any);
    _update: never | ((id: string, data: any, params: any) => any);
    _patch: never | ((id: string, data: any, params: any) => any);
    _remove: never | ((id: string, params: any) => any);
}
type FindMethod<P, R> = (params: P) => Promise<R[] | Stripe.ApiList<R>>;

type FilteredParams<T extends ParamsWithStripe = ParamsWithStripe> = {
    query: T["query"];
    stripe: T["stripe"];
    paginate: boolean;
};
declare abstract class BaseService<I extends IUnderScoreFunctions, Find extends I["_find"] = I["_find"], Get extends I["_get"] = I["_get"], Create extends I["_create"] = I["_create"], Update extends I["_update"] = I["_update"], Patch extends I["_patch"] = I["_patch"], Remove extends I["_remove"] = I["_remove"]> {
    stripe: Stripe;
    options: StripeServiceOptions;
    abstract _find(params: Parameters<Find>[0]): ReturnType<Find>;
    abstract _get(id: string, params: Parameters<Get>[1]): ReturnType<Get>;
    abstract _create(data: Parameters<Create>[0], params: Parameters<Create>[1]): ReturnType<Create>;
    abstract _update(id: string, data: Parameters<Update>[1], params: Parameters<Update>[2]): ReturnType<Update>;
    abstract _patch(id: string, data: Parameters<Patch>[1], params: Parameters<Patch>[2]): ReturnType<Patch>;
    abstract _remove(id: string, params: Parameters<Remove>[1]): ReturnType<Remove>;
    constructor(options: StripeServiceOptions);
    find(params: Parameters<Find>[0]): ReturnType<Find>;
    get(id: Parameters<Get>[0], params: Parameters<Get>[1]): ReturnType<Get>;
    create(data: Parameters<Create>[0], params: Parameters<Create>[1]): ReturnType<Create>;
    update(id: Parameters<Update>[0], data: Parameters<Update>[1], params: Parameters<Update>[2]): ReturnType<Update>;
    patch(id: Parameters<Patch>[0], data: Parameters<Patch>[1], params: Parameters<Patch>[2]): ReturnType<Patch>;
    remove(id: Parameters<Remove>[0], params: Parameters<Remove>[1]): ReturnType<Remove>;
    getLimit(limit: number | undefined, paramsPaginate: false | {
        max: number;
    } | undefined): number;
    cleanQuery<Q extends Query | Query[]>(query: Q): Q;
    filterQuery<P extends ParamsWithStripe, Q = P extends Params<infer T> ? T : Query>(params: P): Q;
    filterParams<T extends ParamsWithStripe = ParamsWithStripe>(params: T): FilteredParams<T>;
    handlePaginate<R = any>({ paginate }: FilteredParams, stripeMethod: Stripe.ApiListPromise<R>): Promise<R[] | Stripe.ApiList<R>>;
    handleError(error: Stripe.errors.StripeError): void;
}

interface IAccountService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.AccountListParams>, Stripe.Account>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Account>;
    _create: (data: Stripe.AccountCreateParams, params: ParamsWithStripe) => Promise<Stripe.Account>;
    _update: (id: string, data: Stripe.AccountUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Account>;
    _patch: (id: string, data: Stripe.AccountUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Account>;
    _remove: never;
}
declare class AccountService extends BaseService<IAccountService> implements IAccountService {
    _find(params: ParamsWithStripeQuery<Stripe.AccountListParams>): Promise<Stripe.ApiList<Stripe.Account> | Stripe.Account[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Account>>;
    _create(data: Stripe.AccountCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Account>>;
    _update(id: string, data: Stripe.AccountUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Account>>;
    _patch(id: string, data: Stripe.AccountUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Account>>;
    _remove: never;
}

interface IAccountLinksService {
    _find: never;
    _get: never;
    _create(data: Stripe.AccountLinkCreateParams, params: ParamsWithStripe): Promise<Stripe.AccountLink>;
    _update: never;
    _patch: never;
    _remove: never;
}
declare class AccountLinkService extends BaseService<IAccountLinksService> implements IAccountLinksService {
    _create(data: Stripe.AccountLinkCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.AccountLink>>;
    _find: never;
    _get: never;
    _update: never;
    _patch: never;
    _remove: never;
}

interface IApplicationFeeRefundService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.ApplicationFeeListParams & {
        fee: string;
    }>, Stripe.FeeRefund>;
    _get: (id: string, params: ParamsWithStripeFee) => Promise<Stripe.FeeRefund>;
    _create: (data: Stripe.FeeRefundCreateParams, params: ParamsWithStripeFee) => Promise<Stripe.FeeRefund>;
    _update: (id: string, data: Stripe.FeeRefundUpdateParams, params: ParamsWithStripeFee) => Promise<Stripe.FeeRefund>;
    _patch: (id: string, data: Stripe.FeeRefundUpdateParams, params: ParamsWithStripeFee) => Promise<Stripe.FeeRefund>;
    _remove: never;
}
declare class ApplicationFeeRefundService extends BaseService<IApplicationFeeRefundService> implements IApplicationFeeRefundService {
    _find(params: ParamsWithStripeQuery<Stripe.ApplicationFeeListParams & {
        fee: string;
    }>): Promise<Stripe.ApiList<Stripe.FeeRefund> | Stripe.FeeRefund[]>;
    _get(id: string, params: ParamsWithStripeFee): Promise<Stripe.Response<Stripe.FeeRefund>>;
    _create(data: Stripe.FeeRefundCreateParams, params: ParamsWithStripeFee): Promise<Stripe.Response<Stripe.FeeRefund>>;
    _update(id: string, data: Stripe.FeeRefundUpdateParams, params: ParamsWithStripeFee): Promise<Stripe.Response<Stripe.FeeRefund>>;
    _patch(id: string, data: Stripe.FeeRefundUpdateParams, params: ParamsWithStripeFee): Promise<Stripe.Response<Stripe.FeeRefund>>;
    _remove: never;
}

interface IBalanceService {
    _find: never;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Balance>;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}
declare class BalanceService extends BaseService<IBalanceService> implements IBalanceService {
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Balance>>;
    _find: never;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}

interface IBankAccountService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.CustomerSourceListParams & {
        customer: string;
    }>, Stripe.CustomerSource>;
    _get: (id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource>;
    _create: (data: Stripe.CustomerSourceCreateParams & {
        customer: string;
    }, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource>;
    _update: (id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card | Stripe.Source>;
    _patch: (id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card | Stripe.Source>;
    _remove: (id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>;
}
declare class BankAccountService extends BaseService<IBankAccountService> implements IBankAccountService {
    _find(params: ParamsWithStripeQuery<Stripe.CustomerSourceListParams & {
        customer: string;
    }>): Promise<Stripe.ApiList<Stripe.CustomerSource> | Stripe.CustomerSource[]>;
    _get(id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource>>;
    _create(data: Stripe.CustomerSourceCreateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource>>;
    _update(id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card | Stripe.Source>>;
    _patch(id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card | Stripe.Source>>;
    _remove(id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>>;
}

interface IExternalAccountService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.ExternalAccountListParams & {
        account: string;
    }>, Stripe.BankAccount | Stripe.Card>;
    _get: (id: string, params: ParamsWithStripeQuery<{
        account: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card>;
    _create: (data: Stripe.ExternalAccountCreateParams & {
        account: string;
    }, params: ParamsWithStripe) => Promise<Stripe.BankAccount | Stripe.Card>;
    _update: (id: string, data: Stripe.ExternalAccountUpdateParams, params: ParamsWithStripeQuery<{
        account: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card>;
    _patch: (id: string, data: Stripe.ExternalAccountUpdateParams, params: ParamsWithStripeQuery<{
        account: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card>;
    _remove: (id: string, params: ParamsWithStripeQuery<{
        account: string;
    }>) => Promise<Stripe.DeletedBankAccount | Stripe.DeletedCard>;
}
declare class ExternalAccountService extends BaseService<IExternalAccountService> implements IExternalAccountService {
    _find(params: ParamsWithStripeQuery<Stripe.ExternalAccountListParams & {
        account: string;
    }>): Promise<Stripe.ApiList<Stripe.BankAccount | Stripe.Card> | (Stripe.BankAccount | Stripe.Card)[]>;
    _get(id: string, params: ParamsWithStripeQuery<{
        account: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card>>;
    _create(data: Stripe.ExternalAccountCreateParams & {
        account: string;
    }, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card>>;
    _update(id: string, data: Stripe.ExternalAccountUpdateParams, params: ParamsWithStripeQuery<{
        account: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card>>;
    _patch(id: string, data: Stripe.ExternalAccountUpdateParams, params: ParamsWithStripeQuery<{
        account: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card>>;
    _remove(id: string, params: ParamsWithStripeQuery<{
        account: string;
    }>): Promise<Stripe.Response<Stripe.DeletedBankAccount | Stripe.DeletedCard>>;
}

interface ICardService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.CustomerSourceListParams & {
        customer: string;
    }>, Stripe.CustomerSource>;
    _get: (id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource>;
    _create: (data: Stripe.CustomerSourceCreateParams & {
        customer: string;
    }, params: ParamsWithStripe) => Promise<Stripe.CustomerSource>;
    _update: (id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card | Stripe.Source>;
    _patch: (id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.BankAccount | Stripe.Card | Stripe.Source>;
    _remove: (id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>;
}
declare class CardService extends BaseService<ICardService> implements ICardService {
    _find(params: ParamsWithStripeQuery<Stripe.CustomerSourceListParams & {
        customer: string;
    }>): Promise<Stripe.ApiList<Stripe.CustomerSource> | Stripe.CustomerSource[]>;
    _get(id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource>>;
    _create(data: Stripe.CustomerSourceCreateParams & {
        customer: string;
    }, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.CustomerSource>>;
    _update(id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card | Stripe.Source>>;
    _patch(id: string, data: Stripe.CustomerSourceUpdateParams, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.BankAccount | Stripe.Card | Stripe.Source>>;
    _remove(id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>>;
}

interface IChargeService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.ChargeListParams>, Stripe.Charge>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Charge>;
    _create: (data: Stripe.ChargeCreateParams, params: ParamsWithStripe) => Promise<Stripe.Charge>;
    _update: (id: string, data: Stripe.ChargeUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Charge>;
    _patch: (id: string, data: ({
        capture: boolean;
    } & Stripe.ChargeCaptureParams) | Stripe.ChargeUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Charge>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.Refund>;
}
declare class ChargeService extends BaseService<IChargeService> implements IChargeService {
    _find(params: ParamsWithStripeQuery<Stripe.ChargeListParams>): Promise<Stripe.ApiList<Stripe.Charge> | Stripe.Charge[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Charge>>;
    _create(data: Stripe.ChargeCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Charge>>;
    _update(id: string, data: Stripe.ChargeUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Charge>>;
    _patch(id: string, data: ({
        capture: boolean;
    } & Stripe.ChargeCaptureParams) | Stripe.ChargeUpdateParams, params: ParamsWithStripe): Promise<Stripe.Charge>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Refund>>;
}

interface ICouponService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.CouponListParams>, Stripe.Coupon>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Coupon>;
    _create: (data: Stripe.CouponCreateParams, params: ParamsWithStripe) => Promise<Stripe.Coupon>;
    _update: (id: string, data: Stripe.CouponUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Coupon>;
    _patch: (id: string, data: Stripe.CouponUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Coupon>;
    _remove: never;
}
declare class CouponService extends BaseService<ICouponService> implements ICouponService {
    _find(params: ParamsWithStripeQuery<Stripe.CouponListParams>): Promise<Stripe.ApiList<Stripe.Coupon> | Stripe.Coupon[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Coupon>>;
    _create(data: Stripe.CouponCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Coupon>>;
    _update(id: string, data: Stripe.CouponUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Coupon>>;
    _patch(id: string, data: Stripe.CouponUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Coupon>>;
    _remove: never;
}

interface ICustomerService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.CustomerListParams>, Stripe.Customer>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Customer | Stripe.DeletedCustomer>;
    _create: (data: Stripe.CustomerCreateParams, params: ParamsWithStripe) => Promise<Stripe.Customer>;
    _update: (id: string, data: Stripe.CustomerUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Customer>;
    _patch: (id: string, data: Stripe.CustomerUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Customer>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.DeletedCustomer>;
}
declare class CustomerService extends BaseService<ICustomerService> implements ICustomerService {
    _find(params: ParamsWithStripeQuery<Stripe.CustomerListParams>): Promise<Stripe.ApiList<Stripe.Customer> | Stripe.Customer[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Customer | Stripe.DeletedCustomer>>;
    _create(data: Stripe.CustomerCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Customer>>;
    _update(id: string, data: Stripe.CustomerUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Customer>>;
    _patch(id: string, data: Stripe.CustomerUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Customer>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.DeletedCustomer>>;
}

interface IDisputeService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.DisputeListParams>, Stripe.Dispute>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Dispute>;
    _create: never;
    _update: (id: string, data: Stripe.DisputeUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Dispute>;
    _patch: (id: string, data: Stripe.DisputeUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Dispute>;
    _remove: never;
}
declare class DisputeService extends BaseService<IDisputeService> implements IDisputeService {
    _find(params: ParamsWithStripeQuery<Stripe.DisputeListParams>): Promise<Stripe.ApiList<Stripe.Dispute> | Stripe.Dispute[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Dispute>>;
    _update(id: string, data: Stripe.DisputeUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Dispute>>;
    _patch(id: string, data: Stripe.DisputeUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Dispute>>;
    _create: never;
    _remove: never;
}

interface IEventService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.EventListParams>, Stripe.Event>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Event>;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}
declare class EventService extends BaseService<IEventService> implements IEventService {
    _find(params: ParamsWithStripeQuery<Stripe.EventListParams>): Promise<Stripe.ApiList<Stripe.Event> | Stripe.Event[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Event>>;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}

interface IInvoiceService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.InvoiceListParams>, Stripe.Invoice>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Invoice>;
    _create: (data: Stripe.InvoiceCreateParams, params: ParamsWithStripe) => Promise<Stripe.Invoice>;
    _update: (id: string, data: Stripe.InvoiceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Invoice>;
    _patch: (id: string, data: ({
        pay: boolean;
    } & Stripe.InvoicePayParams) | Stripe.InvoiceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Invoice>;
    _remove: never;
}
declare class InvoiceService extends BaseService<IInvoiceService> implements IInvoiceService {
    _find(params: ParamsWithStripeQuery<Stripe.InvoiceListParams>): Promise<Stripe.ApiList<Stripe.Invoice> | Stripe.Invoice[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Invoice>>;
    _create(data: Stripe.InvoiceCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Invoice>>;
    _update(id: string, data: Stripe.InvoiceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Invoice>>;
    _patch(id: string, data: ({
        pay: boolean;
    } & Stripe.InvoicePayParams) | Stripe.InvoiceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Invoice>;
    _remove: never;
}

interface IInvoiceItemService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.InvoiceItemListParams>, Stripe.InvoiceItem>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.InvoiceItem>;
    _create(data: Stripe.InvoiceItemCreateParams, params: ParamsWithStripe): Promise<Stripe.InvoiceItem>;
    _update: (id: string, data: Stripe.InvoiceItemUpdateParams, params: ParamsWithStripe) => Promise<Stripe.InvoiceItem>;
    _patch: (id: string, data: Stripe.InvoiceItemUpdateParams, params: ParamsWithStripe) => Promise<Stripe.InvoiceItem>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.DeletedInvoiceItem>;
}
declare class InvoiceItemService extends BaseService<IInvoiceItemService> implements IInvoiceItemService {
    _find(params: ParamsWithStripeQuery<Stripe.InvoiceItemListParams>): Promise<Stripe.ApiList<Stripe.InvoiceItem> | Stripe.InvoiceItem[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.InvoiceItem>>;
    _create(data: Stripe.InvoiceItemCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.InvoiceItem>>;
    _update(id: string, data: Stripe.InvoiceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.InvoiceItem>>;
    _patch(id: string, data: Stripe.InvoiceItemUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.InvoiceItem>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.DeletedInvoiceItem>>;
}

interface IPayoutService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.PayoutListParams>, Stripe.Payout>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Payout>;
    _create: (data: Stripe.PayoutCreateParams, params: ParamsWithStripe) => Promise<Stripe.Payout>;
    _update: (id: string, data: Stripe.PayoutUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Payout>;
    _patch: (id: string, data: Stripe.PayoutUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Payout>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.Payout>;
}
declare class PayoutService extends BaseService<IPayoutService> implements IPayoutService {
    _find(params: ParamsWithStripeQuery<Stripe.PayoutListParams>): Promise<Stripe.ApiList<Stripe.Payout> | Stripe.Payout[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Payout>>;
    _create(data: Stripe.PayoutCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Payout>>;
    _update(id: string, data: Stripe.PayoutUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Payout>>;
    _patch(id: string, data: Stripe.PayoutUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Payout>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Payout>>;
}

interface IPaymentIntentService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.PaymentIntentListParams>, Stripe.PaymentIntent>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.PaymentIntent>;
    _create: (data: Stripe.PaymentIntentCreateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentIntent>;
    _update: (id: string, data: Stripe.PaymentIntentUpdateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentIntent>;
    _patch: (id: string, data: Stripe.PaymentIntentUpdateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentIntent>;
    _remove: never;
}
declare class PaymentIntentService extends BaseService<IPaymentIntentService> implements IPaymentIntentService {
    _find(params: ParamsWithStripeQuery<Stripe.PaymentIntentListParams>): Promise<Stripe.ApiList<Stripe.PaymentIntent> | Stripe.PaymentIntent[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentIntent>>;
    _create(data: Stripe.PaymentIntentCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentIntent>>;
    _patch(id: string, data: {
        capture: boolean;
    } | Stripe.PaymentIntentUpdateParams, params: ParamsWithStripe): Promise<Stripe.PaymentIntent>;
    _update(id: string, data: Stripe.PaymentIntentUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentIntent>>;
    _remove: never;
}

interface IPaymentMethodService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.PaymentMethodListParams>, Stripe.PaymentMethod>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>;
    _create: (data: Stripe.PaymentMethodCreateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>;
    _update: (id: string, data: Stripe.PaymentMethodUpdateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>;
    _patch: ((id: string, data: {
        attach: boolean;
    } & Stripe.PaymentMethodAttachParams, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>) | ((id: string, data: Stripe.PaymentMethodUpdateParams, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>);
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.PaymentMethod>;
}
declare class PaymentMethodService extends BaseService<IPaymentMethodService> implements IPaymentMethodService {
    _find(params: ParamsWithStripeQuery<Stripe.PaymentMethodListParams>): Promise<Stripe.ApiList<Stripe.PaymentMethod> | Stripe.PaymentMethod[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentMethod>>;
    _create(data: Stripe.PaymentMethodCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentMethod>>;
    _update(id: string, data: Stripe.PaymentMethodUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentMethod>>;
    _patch(id: string, data: ({
        attach: boolean;
    } & Stripe.PaymentMethodAttachParams) | Stripe.PaymentMethodUpdateParams, params: ParamsWithStripe): Promise<Stripe.PaymentMethod>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.PaymentMethod>>;
}

interface IPlanService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.PlanListParams>, Stripe.Plan>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Plan>;
    _create: (data: Stripe.PlanCreateParams, params: ParamsWithStripe) => Promise<Stripe.Plan>;
    _update: (id: string, data: Stripe.PlanUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Plan>;
    _patch: (id: string, data: Stripe.PlanUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Plan>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.DeletedPlan>;
}
declare class PlanService extends BaseService<IPlanService> implements IPlanService {
    _find(params: ParamsWithStripeQuery<Stripe.PlanListParams>): Promise<Stripe.ApiList<Stripe.Plan> | Stripe.Plan[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Plan>>;
    _create(data: Stripe.PlanCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Plan>>;
    _update(id: string, data: Stripe.PlanUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Plan>>;
    _patch(id: string, data: Stripe.PlanUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Plan>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.DeletedPlan>>;
}

interface IPriceService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.PriceListParams>, Stripe.Price>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Price>;
    _create: (data: Stripe.PriceCreateParams, params: ParamsWithStripe) => Promise<Stripe.Price>;
    _update: (id: string, data: Stripe.PriceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Price>;
    _patch: (id: string, data: Stripe.PriceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Price>;
    _remove: never;
}
declare class PriceService extends BaseService<IPriceService> implements IPriceService {
    _find(params: ParamsWithStripeQuery<Stripe.PriceListParams>): Promise<Stripe.ApiList<Stripe.Price> | Stripe.Price[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Price>>;
    _create(data: Stripe.PriceCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Price>>;
    _update(id: string, data: Stripe.PriceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Price>>;
    _patch(id: string, data: Stripe.PriceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Price>>;
    _remove: never;
}

interface IProductService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.ProductListParams>, Stripe.Product>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Product>;
    _create: (data: Stripe.ProductCreateParams, params: ParamsWithStripe) => Promise<Stripe.Product>;
    _update: (id: string, data: Stripe.ProductUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Product>;
    _patch: (id: string, data: Stripe.ProductUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Product>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.DeletedProduct>;
}
declare class ProductService extends BaseService<IProductService> implements IProductService {
    _find(params: ParamsWithStripeQuery<Stripe.ProductListParams>): Promise<Stripe.ApiList<Stripe.Product> | Stripe.Product[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Product>>;
    _create(data: Stripe.ProductCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Product>>;
    _update(id: string, data: Stripe.ProductUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Product>>;
    _patch(id: string, data: Stripe.ProductUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Product>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.DeletedProduct>>;
}

interface IRefundService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.RefundListParams>, Stripe.Refund>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Refund>;
    _create: (data: Stripe.RefundCreateParams, params: ParamsWithStripe) => Promise<Stripe.Refund>;
    _update: (id: string, data: Stripe.RefundUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Refund>;
    _patch: (id: string, data: Stripe.RefundUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Refund>;
    _remove: never;
}
declare class RefundService extends BaseService<IRefundService> implements IRefundService {
    _find(params: ParamsWithStripeQuery<Stripe.RefundListParams>): Promise<Stripe.ApiList<Stripe.Refund> | Stripe.Refund[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Refund>>;
    _create(data: Stripe.RefundCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Refund>>;
    _update(id: string, data: Stripe.RefundUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Refund>>;
    _patch(id: string, data: Stripe.RefundUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Refund>>;
    _remove: never;
}

interface ISetupIntentService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.SetupIntentListParams>, Stripe.SetupIntent>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.SetupIntent>;
    _create: (data: Stripe.SetupIntentCreateParams, params: ParamsWithStripe) => Promise<Stripe.SetupIntent>;
    _update: (id: string, data: Stripe.SetupIntentUpdateParams, params: ParamsWithStripe) => Promise<Stripe.SetupIntent>;
    _patch: (id: string, data: Stripe.SetupIntentUpdateParams, params: ParamsWithStripe) => Promise<Stripe.SetupIntent>;
    _remove: never;
}
declare class SetupIntentService extends BaseService<ISetupIntentService> implements ISetupIntentService {
    _find(params: ParamsWithStripeQuery<Stripe.SetupIntentListParams>): Promise<Stripe.ApiList<Stripe.SetupIntent> | Stripe.SetupIntent[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SetupIntent>>;
    _create(data: Stripe.SetupIntentCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SetupIntent>>;
    _update(id: string, data: Stripe.SetupIntentUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SetupIntent>>;
    _patch(id: string, data: Stripe.SetupIntentUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SetupIntent>>;
    _remove: never;
}

interface ISourceService {
    _find: never;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Source>;
    _create: ((data: {
        customer: string;
    } & Stripe.CustomerSourceCreateParams, params: ParamsWithStripe) => Promise<Stripe.CustomerSource>) | ((data: Stripe.SourceCreateParams, params: ParamsWithStripe) => Promise<Stripe.Source>);
    _update: (id: string, data: Stripe.SourceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Source>;
    _patch: (id: string, data: Stripe.SourceUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Source>;
    _remove: (id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>) => Promise<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>;
}
declare class SourceService extends BaseService<ISourceService> implements ISourceService {
    _find: never;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Source>>;
    _create(data: {
        customer: string;
    } & Stripe.CustomerSourceCreateParams, params: ParamsWithStripe): Promise<Stripe.CustomerSource>;
    _create(data: Stripe.SourceCreateParams, params: ParamsWithStripe): Promise<Stripe.Source>;
    _update(id: string, data: Stripe.SourceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Source>>;
    _patch(id: string, data: Stripe.SourceUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Source>>;
    _remove(id: string, params: ParamsWithStripeQuery<{
        customer: string;
    }>): Promise<Stripe.Response<Stripe.CustomerSource | Stripe.DeletedBankAccount | Stripe.DeletedCard>>;
}

interface ISubscriptionService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.SubscriptionListParams>, Stripe.Subscription>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Subscription>;
    _create: (data: Stripe.SubscriptionCreateParams, params: ParamsWithStripe) => Promise<Stripe.Subscription>;
    _update: (id: string, data: Stripe.SubscriptionUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Subscription>;
    _patch: (id: string, data: Stripe.SubscriptionUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Subscription>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.Subscription>;
}
declare class SubscriptionService extends BaseService<ISubscriptionService> implements ISubscriptionService {
    _find(params: ParamsWithStripeQuery<Stripe.SubscriptionListParams>): Promise<Stripe.ApiList<Stripe.Subscription> | Stripe.Subscription[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Subscription>>;
    _create(data: Stripe.SubscriptionCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Subscription>>;
    _update(id: string, data: Stripe.SubscriptionUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Subscription>>;
    _patch(id: string, data: Stripe.SubscriptionUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Subscription>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Subscription>>;
}

interface ISubscriptionItemService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.SubscriptionItemListParams>, Stripe.SubscriptionItem>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.SubscriptionItem>;
    _create: (data: Stripe.SubscriptionItemCreateParams, params: ParamsWithStripe) => Promise<Stripe.SubscriptionItem>;
    _update: (id: string, data: Stripe.SubscriptionItemUpdateParams, params: ParamsWithStripe) => Promise<Stripe.SubscriptionItem>;
    _patch: (id: string, data: Stripe.SubscriptionItemUpdateParams, params: ParamsWithStripe) => Promise<Stripe.SubscriptionItem>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.DeletedSubscriptionItem>;
}
declare class SubscriptionItemService extends BaseService<ISubscriptionItemService> implements ISubscriptionItemService {
    _find(params: ParamsWithStripeQuery<Stripe.SubscriptionItemListParams>): Promise<Stripe.ApiList<Stripe.SubscriptionItem> | Stripe.SubscriptionItem[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SubscriptionItem>>;
    _create(data: Stripe.SubscriptionItemCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SubscriptionItem>>;
    _update(id: string, data: Stripe.SubscriptionItemUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SubscriptionItem>>;
    _patch(id: string, data: Stripe.SubscriptionItemUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.SubscriptionItem>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.DeletedSubscriptionItem>>;
}

interface ITokenService {
    _find: never;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Token>;
    _create: (data: Stripe.TokenCreateParams, params: ParamsWithStripe) => Promise<Stripe.Token>;
    _update: never;
    _patch: never;
    _remove: never;
}
declare class TokenService extends BaseService<ITokenService> implements ITokenService {
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Token>>;
    _create(data: Stripe.TokenCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Token>>;
    _find: never;
    _update: never;
    _patch: never;
    _remove: never;
}

interface ITransactionService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.BalanceTransactionListParams>, Stripe.BalanceTransaction>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.BalanceTransaction>;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}
declare class TransactionService extends BaseService<ITransactionService> implements ITransactionService {
    _find(params: ParamsWithStripeQuery<Stripe.BalanceTransactionListParams>): Promise<Stripe.ApiList<Stripe.BalanceTransaction> | Stripe.BalanceTransaction[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.BalanceTransaction>>;
    _create: never;
    _update: never;
    _patch: never;
    _remove: never;
}

interface ITransferService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.TransferListParams>, Stripe.Transfer>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.Transfer>;
    _create: (data: Stripe.TransferCreateParams, params: ParamsWithStripe) => Promise<Stripe.Transfer>;
    _update: (id: string, data: Stripe.TransferUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Transfer>;
    _patch: (id: string, data: Stripe.TransferUpdateParams, params: ParamsWithStripe) => Promise<Stripe.Transfer>;
    _remove: (id: string, params: ParamsWithStripe) => Promise<Stripe.TransferReversal>;
}
declare class TransferService extends BaseService<ITransferService> implements ITransferService {
    _find(params: ParamsWithStripeQuery<Stripe.TransferListParams>): Promise<Stripe.ApiList<Stripe.Transfer> | Stripe.Transfer[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Transfer>>;
    _create(data: Stripe.TransferCreateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Transfer>>;
    _update(id: string, data: Stripe.TransferUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Transfer>>;
    _patch(id: string, data: Stripe.TransferUpdateParams, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.Transfer>>;
    _remove(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.TransferReversal>>;
}

interface ITransferReversalService {
    _find: FindMethod<ParamsWithStripeQuery<Stripe.TransferReversalListParams & {
        transfer: string;
    }>, Stripe.TransferReversal>;
    _get: (id: string, params: ParamsWithStripe) => Promise<Stripe.TransferReversal>;
    _create: (data: Stripe.TransferReversalCreateParams & {
        transfer: string;
    }, params: ParamsWithStripe) => Promise<Stripe.TransferReversal>;
    _update: (id: string, data: Stripe.TransferReversalUpdateParams, params: ParamsWithStripeQuery<{
        transfer: string;
    }>) => Promise<Stripe.TransferReversal>;
    _patch: (id: string, data: Stripe.TransferReversalUpdateParams, params: ParamsWithStripeQuery<{
        transfer: string;
    }>) => Promise<Stripe.TransferReversal>;
    _remove: never;
}
declare class TransferReversalService extends BaseService<ITransferReversalService> implements ITransferReversalService {
    _find(params: ParamsWithStripeQuery<Stripe.TransferReversalListParams & {
        transfer: string;
    }>): Promise<Stripe.ApiList<Stripe.TransferReversal> | Stripe.TransferReversal[]>;
    _get(id: string, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.TransferReversal>>;
    _create(data: Stripe.TransferReversalCreateParams & {
        transfer: string;
    }, params: ParamsWithStripe): Promise<Stripe.Response<Stripe.TransferReversal>>;
    _update(id: string, data: Stripe.TransferReversalUpdateParams, params: ParamsWithStripeQuery<{
        transfer: string;
    }>): Promise<Stripe.Response<Stripe.TransferReversal>>;
    _patch(id: string, data: Stripe.TransferReversalUpdateParams, params: ParamsWithStripeQuery<{
        transfer: string;
    }>): Promise<Stripe.Response<Stripe.TransferReversal>>;
    _remove: never;
}

interface IWebhookService {
    _find: never;
    _get: never;
    _create: (data: any, params: any) => Promise<any>;
    _update: never;
    _patch: never;
    _remove: never;
}
type WebHookHandler = (options: {
    object: Stripe.Event.Data.Object;
    event: Stripe.Event;
    params: Params;
    app: Application;
}) => any;
interface WebHookHandlers {
    [key: string]: {
        [key: string]: WebHookHandler;
    };
}
interface WebHookOptions {
    app: Application;
    handlers: WebHookHandlers;
    stripe: Stripe;
}
declare class WebhookService extends BaseService<IWebhookService> implements IWebhookService {
    app: Application;
    handlers: WebHookHandlers;
    options: WebHookOptions;
    isExpressSetup: boolean;
    constructor(options: WebHookOptions);
    /**
     * registers an express route
     * @param route
     * @param endpointSecret
     * @returns
     */
    setupExpress(route: string, endpointSecret: string): void;
    getHandler(event: Stripe.Event): WebHookHandler | null;
    _create(event: Stripe.Event, params: Params): any;
    _find: never;
    _get: never;
    _update: never;
    _patch: never;
    _remove: never;
}
type SetupWebHookOptions = {
    endpointSecret: string;
    stripe: Stripe;
    handlers: WebHookHandlers;
};
declare function setupWebhook(app: Application, route: string, { endpointSecret, stripe, handlers }: SetupWebHookOptions): WebhookService;

export { AccountLinkService, AccountService, ApplicationFeeRefundService, BalanceService, BankAccountService, CardService, ChargeService, CouponService, CustomerService, DisputeService, EventService, ExternalAccountService, type FindMethod, type IUnderScoreFunctions, type IWebhookService, InvoiceItemService, InvoiceService, type ParamsWithStripe, type ParamsWithStripeFee, type ParamsWithStripeQuery, PaymentIntentService, PaymentMethodService, PayoutService, PlanService, PriceService, ProductService, RefundService, SetupIntentService, type SetupWebHookOptions, SourceService, type StripeServiceOptions, SubscriptionItemService, SubscriptionService, TokenService, TransactionService, TransferReversalService, TransferService, type WebHookHandler, type WebHookHandlers, type WebHookOptions, WebhookService, setupWebhook };
